@@ -44,6 +44,9 @@ import org.apache.tools.bzip2.CBZip2OutputStream;
  * @author Tim Fennell
  */
 public class IoUtil extends net.sf.samtools.util.IOUtil {
+    /** Possible extensions for VCF files and related formats. */
+    public static final String[] VCF_EXTENSIONS = new String[] {".vcf", ".vcf.gz", ".bcf"};
+
     /**
      * Checks that a file is non-null, exists, is not a directory and is readable.  If any
      * condition is false then a runtime exception is thrown.
@@ -285,7 +288,7 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
      * Preferred over PrintStream and PrintWriter because an exception is thrown on I/O error
      */
     public static BufferedWriter openFileForBufferedWriting(final File file, final boolean append) {
-        return new BufferedWriter(new OutputStreamWriter(openFileForWriting(file, append)), Defaults.BUFFER_SIZE);
+        return new BufferedWriter(new OutputStreamWriter(openFileForWriting(file, append)), Defaults.NON_ZERO_BUFFER_SIZE);
     }
 
     /**
@@ -299,8 +302,8 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
      * Preferred over PrintStream and PrintWriter because an exception is thrown on I/O error
      */
     public static BufferedWriter openFileForBufferedUtf8Writing(final File file) {
-        return new BufferedWriter(new OutputStreamWriter(
-            openFileForWriting(file), Charset.forName("UTF-8")), Defaults.BUFFER_SIZE);
+        return new BufferedWriter(new OutputStreamWriter(openFileForWriting(file), Charset.forName("UTF-8")),
+                Defaults.NON_ZERO_BUFFER_SIZE);
     }
 
     /**
@@ -323,9 +326,13 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
     public static OutputStream openGzipFileForWriting(final File file, final boolean append) {
 
         try {
+            if (Defaults.BUFFER_SIZE > 0) {
             return new CustomGzipOutputStream(new FileOutputStream(file, append),
                                               Defaults.BUFFER_SIZE,
                                               Defaults.COMPRESSION_LEVEL);
+            } else {
+                return new CustomGzipOutputStream(new FileOutputStream(file, append), Defaults.COMPRESSION_LEVEL);
+            }
         }
         catch (IOException ioe) {
             throw new PicardException("Error opening file for writing: " + file.getName(), ioe);
@@ -346,7 +353,7 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
             final FileOutputStream fos = new FileOutputStream(file, append);
             fos.write(66); //write magic number 'BZ' because CBZip2OutputStream does not do it for you
             fos.write(90);
-            return new BufferedOutputStream(new CBZip2OutputStream(fos), Defaults.BUFFER_SIZE);
+            return IOUtil.maybeBufferOutputStream(new CBZip2OutputStream(fos));
         }
         catch (IOException ioe) {
             throw new PicardException("Error opening file for writing: " + file.getName(), ioe);
@@ -366,7 +373,7 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
      */
     public static void copyStream(final InputStream input, final OutputStream output) {
         try {
-            final byte[] buffer = new byte[Defaults.BUFFER_SIZE];
+            final byte[] buffer = new byte[Defaults.NON_ZERO_BUFFER_SIZE];
             int bytesRead = 0;
             while((bytesRead = input.read(buffer)) > 0) {
                 output.write(buffer, 0, bytesRead);
@@ -484,8 +491,8 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
     }
 
     /** Checks that a file exists and is readable, and then returns a buffered reader for it. */
-    public static BufferedReader openFileForBufferedReading(final File file) throws IOException {
-        return new BufferedReader(new InputStreamReader(openFileForReading(file)), Defaults.BUFFER_SIZE);
+    public static BufferedReader openFileForBufferedReading(final File file) {
+        return new BufferedReader(new InputStreamReader(openFileForReading(file)), Defaults.NON_ZERO_BUFFER_SIZE);
 	}
 
     /** Takes a string and replaces any characters that are not safe for filenames with an underscore */
@@ -493,18 +500,6 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
         return str.trim().replaceAll("[\\s!\"#$%&'()*/:;<=>?@\\[\\]\\\\^`{|}~]", "_");
     }
 
-    /** Returns the name of the file minus the extension (i.e. text after the last "." in the filename). */
-    public static String basename(final File f) {
-        final String full = f.getName();
-        final int index = full.lastIndexOf(".");
-        if (index > 0  && index > full.lastIndexOf(File.separator)) {
-            return full.substring(0, index);
-        }
-        else {
-            return full;
-        }
-    }
-
     /** Returns the name of the file extension (i.e. text after the last "." in the filename) including the . */
     public static String fileSuffix(final File f) {
         final String full = f.getName();
@@ -527,7 +522,7 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
                 if (f != null) f = f.getCanonicalFile();
             }
             return canonicalPath;
-        } catch (IOException ioe) {
+        } catch (final IOException ioe) {
             throw new RuntimeException("Error getting full canonical path for " +
                     file + ": " + ioe.getMessage(), ioe);
         }
@@ -538,7 +533,7 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
      */
     public static String readFully(final InputStream in) {
         try {
-            final BufferedReader r = new BufferedReader(new InputStreamReader(in), Defaults.BUFFER_SIZE);
+            final BufferedReader r = new BufferedReader(new InputStreamReader(in), Defaults.NON_ZERO_BUFFER_SIZE);
             final StringBuilder builder = new StringBuilder(512);
             String line = null;
 
@@ -549,7 +544,7 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
 
             return builder.toString();
         }
-        catch (IOException ioe) {
+        catch (final IOException ioe) {
             throw new RuntimeIOException("Error reading stream", ioe);
         }
     }
@@ -579,14 +574,14 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
                         if (next == null) in.close();
                         return tmp;
                     }
-                    catch (IOException ioe) { throw new RuntimeIOException(ioe); }
+                    catch (final IOException ioe) { throw new RuntimeIOException(ioe); }
                 }
 
                 /** Closes the underlying input stream. Not required if end of stream has already been hit. */
                 @Override public void close() throws IOException { CloserUtil.close(in); }
             };
         }
-        catch (IOException e) {
+        catch (final IOException e) {
             throw new RuntimeIOException(e);
         }
     }
@@ -642,17 +637,20 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
         final List<File> output = new ArrayList<File>();
         stack.addAll(inputs);
 
-        final Set<String> exts = new HashSet<String>();
-        Collections.addAll(exts, extensions);
-
         while (!stack.empty()) {
             final File f = stack.pop();
-            final String ext = IoUtil.fileSuffix(f);
+            final String name = f.getName();
+            boolean matched = false;
 
-            if (exts.contains(ext)) {
-                output.add(f);
+            for (final String ext : extensions) {
+                if (!matched && name.endsWith(ext)) {
+                    output.add(f);
+                    matched = true;
+                }
             }
-            else {
+
+            // If the file didn't match a given extension, treat it as a list of files
+            if (!matched) {
                 IoUtil.assertFileIsReadable(f);
 
                 for (final String s : IoUtil.readLines(f)) {
@@ -661,6 +659,9 @@ public class IoUtil extends net.sf.samtools.util.IOUtil {
             }
         }
 
+        // Preserve input order (since we're using a stack above) for things that care
+        Collections.reverse(output);
+
         return output;
     }
 }
@@ -676,5 +677,10 @@ class CustomGzipOutputStream extends GZIPOutputStream {
         super(outputStream, bufferSize);
         this.def.setLevel(compressionLevel);
     }
+
+    CustomGzipOutputStream(final OutputStream outputStream, final int compressionLevel) throws IOException {
+        super(outputStream);
+        this.def.setLevel(compressionLevel);
+    }
 }
 
