@@ -77,6 +77,10 @@ import java.util.List;
  * By default, extensive validation of SAMRecords is done when they are read.  Very limited validation is done when
  * values are set onto SAMRecords.
  */
+/**
+ * @author alecw@broadinstitute.org
+ * @author mishali.naik@intel.com
+ */
 public class SAMRecord implements Cloneable
 {
     /**
@@ -109,7 +113,7 @@ public class SAMRecord implements Cloneable
     /**
      * If a read has reference name "*", it will have this value for position.
      */
-    public static final int NO_ALIGNMENT_START = 0;
+    public static final int NO_ALIGNMENT_START = GenomicIndexUtil.UNSET_GENOMIC_LOCATION;
 
     /**
      * This should rarely be used, since a read with no sequence doesn't make much sense.
@@ -363,7 +367,7 @@ public class SAMRecord implements Cloneable
         } else {
             try {
                 mReferenceName = mHeader.getSequence(referenceIndex).getSequenceName();
-            } catch (NullPointerException e) {
+            } catch (final NullPointerException e) {
                 throw new IllegalArgumentException("Reference index " + referenceIndex + " not found in sequence dictionary.", e);
             }
         }
@@ -422,7 +426,7 @@ public class SAMRecord implements Cloneable
         } else {
             try {
                 mMateReferenceName = mHeader.getSequence(referenceIndex).getSequenceName();
-            } catch (NullPointerException e) {
+            } catch (final NullPointerException e) {
                 throw new IllegalArgumentException("Reference index " + referenceIndex + " not found in sequence dictionary.", e);
             }
         }
@@ -468,19 +472,7 @@ public class SAMRecord implements Cloneable
      * Invalid to call on an unmapped read.
      */
     public int getUnclippedStart() {
-        int pos = getAlignmentStart();
-
-        for (final CigarElement cig : getCigar().getCigarElements()) {
-            final CigarOperator op = cig.getOperator();
-            if (op == CigarOperator.SOFT_CLIP || op == CigarOperator.HARD_CLIP) {
-                pos -= cig.getLength();
-            }
-            else {
-                break;
-            }
-        }
-
-        return pos;
+        return SAMUtils.getUnclippedStart(getAlignmentStart(), getCigar());
     }
 
     /**
@@ -491,23 +483,10 @@ public class SAMRecord implements Cloneable
      * Invalid to call on an unmapped read.
      */
     public int getUnclippedEnd() {
-        int pos = getAlignmentEnd();
-        final List<CigarElement> cigs = getCigar().getCigarElements();
-        for (int i=cigs.size() - 1; i>=0; --i) {
-            final CigarElement cig = cigs.get(i);
-            final CigarOperator op = cig.getOperator();
-
-            if (op == CigarOperator.SOFT_CLIP || op == CigarOperator.HARD_CLIP) {
-                pos += cig.getLength();
-            }
-            else {
-                break;
-            }
-        }
-
-        return pos;               
+        return SAMUtils.getUnclippedEnd(getAlignmentEnd(), getCigar());
     }
 
+
     /**
      * @return 1-based inclusive reference position of the unclipped sequence at a given offset,
      *         or 0 if there is no position.
@@ -603,7 +582,7 @@ public class SAMRecord implements Cloneable
             mCigar = TextCigarCodec.getSingleton().decode(mCigarString);
             if (getValidationStringency() != SAMFileReader.ValidationStringency.SILENT && !this.getReadUnmappedFlag()) {
                 // Don't know line number, and don't want to force read name to be decoded.
-                SAMUtils.processValidationErrors(validateCigar(-1L), -1L, getValidationStringency());
+                SAMUtils.processValidationErrors(this.validateCigar(-1L), -1L, getValidationStringency());
             }
         }
         return mCigar;
@@ -763,8 +742,8 @@ public class SAMRecord implements Cloneable
      */
     public boolean getSupplementaryAlignmentFlag() {
         return (mFlags & SUPPLEMENTARY_ALIGNMENT_FLAG) != 0;
-    }    
-    
+    }
+
     /**
      * the read fails platform/vendor quality checks.
      */
@@ -881,7 +860,7 @@ public class SAMRecord implements Cloneable
     public boolean isSecondaryOrSupplementary() {
         return getNotPrimaryAlignmentFlag() || getSupplementaryAlignmentFlag();
     }
-    
+
     private void setFlag(final boolean flag, final int bit) {
         if (flag) {
             mFlags |= bit;
@@ -1160,8 +1139,8 @@ public class SAMRecord implements Cloneable
     protected void setAttribute(final short tag, final Object value, final boolean isUnsignedArray) {
         if (value != null &&
                 !(value instanceof Byte || value instanceof Short || value instanceof Integer ||
-                value instanceof String || value instanceof Character || value instanceof Float ||
-                value instanceof byte[] || value instanceof short[] || value instanceof int[] ||
+                        value instanceof String || value instanceof Character || value instanceof Float ||
+                        value instanceof byte[] || value instanceof short[] || value instanceof int[] ||
                         value instanceof float[])) {
             throw new SAMException("Attribute type " + value.getClass() + " not supported. Tag: " +
                     SAMTagUtil.getSingleton().makeStringTag(tag));
@@ -1260,7 +1239,7 @@ public class SAMRecord implements Cloneable
             // then treat this as a one base alignment for indexing purposes.
             alignmentEnd = alignmentStart + 1;
         }
-        return SAMUtils.reg2bin(alignmentStart, alignmentEnd);
+        return GenomicIndexUtil.reg2bin(alignmentStart, alignmentEnd);
     }
 
     public SAMFileHeader getHeader() {
@@ -1345,7 +1324,7 @@ public class SAMRecord implements Cloneable
         if (value == null || value instanceof String) {
             return tagString + ":Z:" + value;
         } else if (value instanceof Integer || value instanceof Long ||
-                   value instanceof Short || value instanceof Byte) {
+                value instanceof Short || value instanceof Byte) {
             return tagString + ":i:" + value;
         } else if (value instanceof Character) {
             return tagString + ":A:" + value;
@@ -1355,7 +1334,7 @@ public class SAMRecord implements Cloneable
             return tagString + ":H:" + StringUtil.bytesToHexString((byte[]) value);
         } else {
             throw new RuntimeException("Unexpected value type for tag " + tagString +
-                                       ": " + value + " of class " + value.getClass().getName());
+                    ": " + value + " of class " + value.getClass().getName());
         }
     }
 
@@ -1384,37 +1363,9 @@ public class SAMRecord implements Cloneable
      * deleted bases (vs. the reference) are not represented in the alignment blocks.
      */
     public List<AlignmentBlock> getAlignmentBlocks() {
-        if (this.mAlignmentBlocks != null) return this.mAlignmentBlocks;
-
-        final Cigar cigar = getCigar();
-        if (cigar == null) return Collections.emptyList();
-
-
-        final List<AlignmentBlock> alignmentBlocks = new ArrayList<AlignmentBlock>();
-        int readBase = 1;
-        int refBase  = getAlignmentStart();
-
-        for (final CigarElement e : cigar.getCigarElements()) {
-            switch (e.getOperator()) {
-                case H : break; // ignore hard clips
-                case P : break; // ignore pads
-                case S : readBase += e.getLength(); break; // soft clip read bases
-                case N : refBase += e.getLength(); break;  // reference skip
-                case D : refBase += e.getLength(); break;
-                case I : readBase += e.getLength(); break;
-                case M :
-                case EQ :
-                case X :
-                    final int length = e.getLength();
-                    alignmentBlocks.add(new AlignmentBlock(readBase, refBase, length));
-                    readBase += length;
-                    refBase  += length;
-                    break;
-                default : throw new IllegalStateException("Case statement didn't deal with cigar op: " + e.getOperator());
-            }
+        if (this.mAlignmentBlocks == null) {
+            this.mAlignmentBlocks = SAMUtils.getAlignmentBlocks(getCigar(), getAlignmentStart(), "read cigar");
         }
-        this.mAlignmentBlocks = Collections.unmodifiableList(alignmentBlocks);
-
         return this.mAlignmentBlocks;
     }
 
@@ -1428,20 +1379,7 @@ public class SAMRecord implements Cloneable
         List<SAMValidationError> ret = null;
 
         if (getValidationStringency() != SAMFileReader.ValidationStringency.SILENT && !this.getReadUnmappedFlag()) {
-            // Don't know line number, and don't want to force read name to be decoded.
-            ret = getCigar().isValid(getReadName(), recordNumber);
-            if (getReferenceIndex() != NO_ALIGNMENT_REFERENCE_INDEX) {
-                final SAMSequenceRecord sequence = getHeader().getSequence(getReferenceIndex());
-                final int referenceSequenceLength = sequence.getSequenceLength();
-                for (final AlignmentBlock alignmentBlock : getAlignmentBlocks()) {
-                    if (alignmentBlock.getReferenceStart() + alignmentBlock.getLength() - 1 > referenceSequenceLength) {
-                        if (ret == null) ret = new ArrayList<SAMValidationError>();
-                        ret.add(new SAMValidationError(SAMValidationError.Type.CIGAR_MAPS_OFF_REFERENCE,
-                                "CIGAR M operator maps off end of reference", getReadName(), recordNumber));
-                        break;
-                    }
-                }
-            }
+            ret = SAMUtils.validateCigar(this, getCigar(), getReferenceIndex(), getAlignmentBlocks(), recordNumber, "Read CIGAR");
         }
         return ret;
     }
@@ -1637,7 +1575,7 @@ public class SAMRecord implements Cloneable
             ret.addAll(errors);
         }
         // TODO(mccowan): Is this asking "is this the primary alignment"?
-        if (this.getReadLength() == 0 && !this.getNotPrimaryAlignmentFlag()) {  
+        if (this.getReadLength() == 0 && !this.getNotPrimaryAlignmentFlag()) {
             final Object fz = getAttribute(SAMTagUtil.getSingleton().FZ);
             if (fz == null) {
                 final String cq = (String)getAttribute(SAMTagUtil.getSingleton().CQ);
@@ -1701,7 +1639,7 @@ public class SAMRecord implements Cloneable
     }
 
     private List<SAMValidationError> isValidReferenceIndexAndPosition(final Integer referenceIndex, final String referenceName,
-                                                          final int alignmentStart, final boolean isMate) {
+                                                                      final int alignmentStart, final boolean isMate) {
         final boolean hasReference = hasReferenceName(referenceIndex, referenceName);
 
         // ret is only instantiate if there are errors to report, in order to reduce GC in the typical case
@@ -1735,7 +1673,7 @@ public class SAMRecord implements Cloneable
         }
         return ret;
     }
-    
+
     private String buildMessage(final String baseMessage, final boolean isMate) {
         return isMate ? "Mate " + baseMessage : baseMessage;
     }
@@ -1758,7 +1696,7 @@ public class SAMRecord implements Cloneable
     /** Simple toString() that gives a little bit of useful info about the read. */
     @Override
     public String toString() {
-        StringBuilder builder = new StringBuilder(64);
+        final StringBuilder builder = new StringBuilder(64);
         builder.append(getReadName());
         if (getReadPairedFlag()) {
             if (getFirstOfPairFlag()) {
@@ -1784,11 +1722,11 @@ public class SAMRecord implements Cloneable
     }
 
     /**
-	Returns the record in the SAM line-based text format.  Fields are
-	separated by '\t' characters, and the String is terminated by '\n'.
-    */
+     Returns the record in the SAM line-based text format.  Fields are
+     separated by '\t' characters, and the String is terminated by '\n'.
+     */
     public String getSAMString() {
-	return SAMTextWriter.getSAMString(this);
+        return SAMTextWriter.getSAMString(this);
     }
 }
 
