@@ -35,6 +35,7 @@ import java.nio.ByteOrder;
 import java.util.Arrays;
 
 import net.sf.samtools.FileTruncatedException;
+import net.sf.samtools.SAMException;
 import net.sf.samtools.seekablestream.SeekableBufferedStream;
 import net.sf.samtools.seekablestream.SeekableFileStream;
 import net.sf.samtools.seekablestream.SeekableHTTPStream;
@@ -48,7 +49,7 @@ import net.sf.samtools.seekablestream.SeekableStream;
  *
  * c.f. http://samtools.sourceforge.net/SAM1.pdf for details of BGZF format
  */
-public class BlockCompressedInputStream extends InputStream {
+public class BlockCompressedInputStream extends InputStream implements LocationAware {
     private InputStream mStream = null;
     private SeekableStream mFile = null;
     private byte[] mFileBuffer = null;
@@ -63,7 +64,20 @@ public class BlockCompressedInputStream extends InputStream {
      * Note that seek() is not supported if this ctor is used.
      */
     public BlockCompressedInputStream(final InputStream stream) {
-        mStream = IOUtil.toBufferedStream(stream);
+        this(stream, true);
+    }
+
+    /**
+     * Note that seek() is not supported if this ctor is used.
+     */
+    public BlockCompressedInputStream(final InputStream stream, final boolean allowBuffering) {
+        if (allowBuffering) {
+            mStream = IOUtil.toBufferedStream(stream);
+        }
+        else {
+            mStream = stream;
+        }
+
         mFile = null;
     }
 
@@ -307,6 +321,11 @@ public class BlockCompressedInputStream extends InputStream {
         return BlockCompressedFilePointerUtil.makeFilePointer(mBlockAddress, mCurrentOffset);
     }
 
+    @Override
+    public long getPosition() {
+        return getFilePointer();
+    }
+
     public static long getFileBlock(final long bgzfOffset) {
         return BlockCompressedFilePointerUtil.getBlockAddress(bgzfOffset);
     }
@@ -376,7 +395,7 @@ public class BlockCompressedInputStream extends InputStream {
         if (buffer == null || buffer.length != uncompressedLength) {
             try {
                 buffer = new byte[uncompressedLength];
-            } catch (NegativeArraySizeException e) {
+            } catch (final NegativeArraySizeException e) {
                 throw new RuntimeException("BGZF file has invalid uncompressedLength: " + uncompressedLength, e);
             }
         }
@@ -474,6 +493,12 @@ public class BlockCompressedInputStream extends InputStream {
         }
     }
 
+    public static void assertNonDefectiveFile(final File file) throws IOException {
+        if (checkTermination(file) == FileTermination.DEFECTIVE) {
+            throw new SAMException(file.getAbsolutePath() + " does not have a valid GZIP block at the end of the file.");
+        }
+    }
+
     private static boolean preambleEqual(final byte[] preamble, final byte[] buf, final int startOffset, final int length) {
         for (int i = 0; i < length; ++i) {
             if (preamble[i] != buf[i + startOffset]) {
