@@ -23,26 +23,33 @@
  */
 package org.broad.tribble.index;
 
+import net.sf.samtools.Defaults;
+import net.sf.samtools.SAMSequenceDictionary;
+import net.sf.samtools.util.BlockCompressedInputStream;
 import org.broad.tribble.*;
 import org.broad.tribble.index.interval.IntervalIndexCreator;
 import org.broad.tribble.index.interval.IntervalTreeIndex;
 import org.broad.tribble.index.linear.LinearIndex;
 import org.broad.tribble.index.linear.LinearIndexCreator;
-import org.broad.tribble.readers.LocationAware;
+import org.broad.tribble.index.tabix.TabixFormat;
+import org.broad.tribble.index.tabix.TabixIndex;
+import net.sf.samtools.util.LocationAware;
+import org.broad.tribble.index.tabix.TabixIndexCreator;
 import org.broad.tribble.readers.PositionalBufferedStream;
 import org.broad.tribble.util.LittleEndianInputStream;
 import org.broad.tribble.util.LittleEndianOutputStream;
 import org.broad.tribble.util.ParsingUtils;
+import org.broad.tribble.util.TabixUtils;
 
 import java.io.*;
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.Map;
+import java.lang.reflect.Constructor;
+import java.util.*;
 import java.util.zip.GZIPInputStream;
 
 /**
  * Factory class for creating indexes.  It is the responsibility of this class to determine and create the
- * correct index type from the input file or stream
+ * correct index type from the input file or stream.  Only LinearIndex and IntervalTreeIndex are supported
+ * by this factory.
  */
 public class IndexFactory {
     /** We can optimize index-file-creation for different factors. As of this writing, those are index-file size or seeking time. */
@@ -55,11 +62,13 @@ public class IndexFactory {
      * an enum that contains all of the information about the index types, and how to create them
      */
     public enum IndexType {
-        LINEAR(1, LinearIndexCreator.class, LinearIndex.class, LinearIndexCreator.DEFAULT_BIN_WIDTH),
-        INTERVAL_TREE(2, IntervalIndexCreator.class, IntervalTreeIndex.class, IntervalIndexCreator.DEFAULT_FEATURE_COUNT),
-        TABIX(3, null, null, -1);
+        LINEAR(LinearIndex.MAGIC_NUMBER, LinearIndex.INDEX_TYPE, LinearIndexCreator.class, LinearIndex.class, LinearIndexCreator.DEFAULT_BIN_WIDTH),
+        INTERVAL_TREE(IntervalTreeIndex.MAGIC_NUMBER, IntervalTreeIndex.INDEX_TYPE, IntervalIndexCreator.class, IntervalTreeIndex.class, IntervalIndexCreator.DEFAULT_FEATURE_COUNT),
+        // Tabix index initialization requires additional information, so generic construction won't work, thus indexCreatorClass is null.
+        TABIX(TabixIndex.MAGIC_NUMBER, null, null, TabixIndex.class, -1);
 
-        private final int indexValue;
+        private final int magicNumber;
+        private final Integer tribbleIndexType;
         private final Class<IndexCreator> indexCreatorClass;
         private final int defaultBinSize;
         private final Class<Index> indexType;
@@ -71,9 +80,9 @@ public class IndexFactory {
         public IndexCreator getIndexCreator() {
             try {
                 return indexCreatorClass.newInstance();
-            } catch ( InstantiationException e ) {
+            } catch ( final InstantiationException e ) {
                 throw new TribbleException("Couldn't make index creator in " + this, e);
-            } catch ( IllegalAccessException e ) {
+            } catch ( final IllegalAccessException e ) {
                 throw new TribbleException("Couldn't make index creator in " + this, e);
             }
         }
@@ -82,31 +91,56 @@ public class IndexFactory {
             return indexCreatorClass != null;
         }
 
-        IndexType(final int headerValue, final Class creator, final Class indexClass, final int defaultBinSize) {
-            indexValue = headerValue;
+        IndexType(final int magicNumber, final Integer tribbleIndexType, final Class creator, final Class indexClass, final int defaultBinSize) {
+            this.magicNumber = magicNumber;
+            this.tribbleIndexType = tribbleIndexType;
             indexCreatorClass = creator;
             indexType = indexClass;
             this.defaultBinSize = defaultBinSize;
         }
 
-        public int getHeaderValue() {
-            return indexValue;
+        public Integer getTribbleIndexType() {
+            return tribbleIndexType;
         }
 
         public Class getIndexType() {
             return indexType;
         }
 
+        public int getMagicNumber() { return magicNumber; }
+
         /**
          *
-         * @param headerValue
+         * @param is InputStream of index.  This will be reset to location it was at when method was invoked.
          * @return The {@code IndexType} based on the {@code headerValue}
          * @throws TribbleException.UnableToCreateCorrectIndexType
          */
-        public static IndexType getIndexType(final int headerValue) {
-            for (final IndexType type : IndexType.values())
-                if (type.indexValue == headerValue) return type;
-            throw new TribbleException.UnableToCreateCorrectIndexType("Unknown index type value" + headerValue);
+        public static IndexType getIndexType(final BufferedInputStream is) {
+            // Currently only need 8 bytes, so this should be plenty
+            is.mark(128);
+            final LittleEndianInputStream dis = new LittleEndianInputStream(is);
+            final int magicNumber;
+            final int type;
+
+            try {
+                // Read the type and version,  then create the appropriate type
+                magicNumber = dis.readInt();
+                // This is not appropriate for all types, but it doesn't hurt to read it.
+                type = dis.readInt();
+                is.reset();
+
+                for (final IndexType indexType : IndexType.values()) {
+                    if (indexType.magicNumber == magicNumber &&
+                            (indexType.tribbleIndexType == null || indexType.tribbleIndexType == type)) {
+                        return indexType;
+                    }
+                }
+            } catch (final IOException e) {
+                throw new TribbleException("Problem detecting index type", e);
+            }
+
+            throw new TribbleException.UnableToCreateCorrectIndexType(
+                    String.format("Unknown index type.  magic number: 0x%x; type %d", magicNumber, type));
         }
     }
 
@@ -116,42 +150,39 @@ public class IndexFactory {
      * at run time by reading the type flag in the file.
      *
      * @param indexFile from which to load the index
-     * @return the parsed index object
      */
     public static Index loadIndex(final String indexFile) {
-        Index idx = null;
-        InputStream is = null;
-        LittleEndianInputStream dis = null;
+        final Index idx = null;
+        BufferedInputStream bufferedInputStream = null;
+        final LittleEndianInputStream dis = null;
         try {
+            InputStream  inputStream = ParsingUtils.openInputStream(indexFile);
             if (indexFile.endsWith(".gz")) {
-                is = new BufferedInputStream(new GZIPInputStream(ParsingUtils.openInputStream(indexFile)), 512000);
-            } else {
-                is = new BufferedInputStream(ParsingUtils.openInputStream(indexFile), 512000);
+                inputStream = new GZIPInputStream(inputStream);
             }
+            else if (indexFile.endsWith(TabixUtils.STANDARD_INDEX_EXTENSION)) {
+                inputStream = new BlockCompressedInputStream(inputStream);
+            }
+            // Must be buffered, because getIndexType uses mark and reset
+            bufferedInputStream = new BufferedInputStream(inputStream, Defaults.NON_ZERO_BUFFER_SIZE);
+            final Class indexClass = IndexType.getIndexType(bufferedInputStream).getIndexType();
 
-            dis = new LittleEndianInputStream(is);
+            final Constructor ctor = indexClass.getConstructor(InputStream.class);
 
-            // Read the type and version,  then create the appropriate type
-            final int magicNumber = dis.readInt();
-            final int type = dis.readInt();
-            final Class indexClass = IndexType.getIndexType(type).getIndexType();
-
-            idx = (Index) indexClass.newInstance();
-            idx.read(dis);
-        } catch (IOException ex) {
+            return (Index) ctor.newInstance(bufferedInputStream);
+        } catch (final IOException ex) {
             throw new TribbleException.UnableToReadIndexFile("Unable to read index file", indexFile, ex);
-        } catch (Exception ex) {
+        } catch (final Exception ex) {
             throw new RuntimeException(ex);
         } finally {
             try {
-                if (is != null) is.close();
+                if (bufferedInputStream != null) bufferedInputStream.close();
                 if (dis != null) dis.close();
                 //log.info(String.format("Closed %s and %s", is, dis));
-            } catch (IOException e) {
+            } catch (final IOException e) {
                 //log.error("Error closing indexFile: " + indexFile, e);
             }
         }
-        return idx;
     }
 
 
@@ -160,23 +191,23 @@ public class IndexFactory {
      *
      * @param inputFile the input file to load features from
      * @param codec     the codec to use for decoding records
-     * @return a index
      */
-    public static Index createLinearIndex(final File inputFile, final FeatureCodec codec) {
-        return createIndex(inputFile, codec, IndexType.LINEAR);
+    public static LinearIndex createLinearIndex(final File inputFile, final FeatureCodec codec) {
+        return createLinearIndex(inputFile, codec, LinearIndexCreator.DEFAULT_BIN_WIDTH);
     }
 
     /**
      * a helper method for creating a linear binned index
      *
-     * @see #createIndex(File, FeatureCodec, IndexType, int)
      * @param inputFile the input file to load features from
      * @param codec     the codec to use for decoding records
      * @param binSize   the bin size
-     * @return a index
      */
-    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createLinearIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec, final int binSize) {
-        return createIndex(inputFile, codec, IndexType.LINEAR, binSize);
+    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> LinearIndex createLinearIndex(final File inputFile,
+                                                                                      final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec,
+                                                                                      final int binSize) {
+        final LinearIndexCreator indexCreator = new LinearIndexCreator(inputFile, binSize);
+        return (LinearIndex)createIndex(inputFile, new FeatureIterator<FEATURE_TYPE, SOURCE_TYPE>(inputFile, codec), indexCreator);
     }
 
     /**
@@ -184,10 +215,10 @@ public class IndexFactory {
      *
      * @param inputFile the file containing the features
      * @param codec to decode the features
-     * @return
      */
-    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createIntervalIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec) {
-        return createIndex(inputFile, codec, IndexType.INTERVAL_TREE);
+    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> IntervalTreeIndex createIntervalIndex(final File inputFile,
+                                                                                        final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec) {
+        return createIntervalIndex(inputFile, codec, IntervalIndexCreator.DEFAULT_FEATURE_COUNT);
     }
 
 
@@ -196,11 +227,13 @@ public class IndexFactory {
      *
      * @param inputFile the input file to load features from
      * @param codec     the codec to use for decoding records
-     * @param binSize   the bin size
-     * @return a index
+     * @param featuresPerInterval
      */
-    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createIntervalIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec, final int binSize) {
-        return createIndex(inputFile, codec, IndexType.INTERVAL_TREE, binSize);
+    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> IntervalTreeIndex createIntervalIndex(final File inputFile,
+                                                                                        final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec,
+                                                                                        final int featuresPerInterval) {
+        final IntervalIndexCreator indexCreator = new IntervalIndexCreator(inputFile, featuresPerInterval);
+        return (IntervalTreeIndex)createIndex(inputFile, new FeatureIterator<FEATURE_TYPE, SOURCE_TYPE>(inputFile, codec), indexCreator);
     }
 
     /**
@@ -208,7 +241,6 @@ public class IndexFactory {
      *
      * @param inputFile the input file to load features from
      * @param codec     the codec to use for decoding records
-     * @return a index
      */
     public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createDynamicIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec) {
         return createDynamicIndex(inputFile, codec, IndexBalanceApproach.FOR_SEEK_TIME);
@@ -220,28 +252,17 @@ public class IndexFactory {
      * @param inputFile the input file to load features from
      * @param codec     the codec to use for decoding records
      * @param type      the type of index to create
-     * @return a index
      */
-    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec, final IndexType type) {
-        return createIndex(inputFile, codec, type, type.getDefaultBinSize());
-    }
-
-    /**
-     * Create a index of the specified type
-     *
-     * @param inputFile the input file to load features from
-     * @param codec     the codec to use for decoding records
-     * @param type      the type of index to create
-     * @param binSize   the bin size
-     * @return a index
-     */
-    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec, final IndexType type, final int binSize) {
-        if ( ! type.canCreate() )
-            throw new TribbleException("Tribble can only read, not create indices of type " + type.name());
-
-        final IndexCreator idx = type.getIndexCreator();
-        idx.initialize(inputFile, binSize);
-        return createIndex(inputFile, new FeatureIterator<FEATURE_TYPE, SOURCE_TYPE>(inputFile, codec), idx);
+    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createIndex(final File inputFile,
+                                                                                final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec,
+                                                                                final IndexType type) {
+        switch (type) {
+            case INTERVAL_TREE: return createIntervalIndex(inputFile, codec);
+            case LINEAR:        return createLinearIndex(inputFile, codec);
+            // Tabix index initialization requires additional information, so this construction method won't work.
+            case TABIX:         throw new UnsupportedOperationException("Tabix indices cannot be created through a generic interface");
+        }
+        throw new IllegalArgumentException("Unrecognized IndexType " + type);
     }
 
     /**
@@ -269,15 +290,31 @@ public class IndexFactory {
      * @param inputFile the input file to load features from
      * @param codec     the codec to use for decoding records
      * @param iba       the index balancing approach
-     * @return a index
      */
-    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createDynamicIndex(final File inputFile, final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec, final IndexBalanceApproach iba) {
+    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> Index createDynamicIndex(final File inputFile,
+                                                                                       final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec,
+                                                                                       final IndexBalanceApproach iba) {
         // get a list of index creators
-        final DynamicIndexCreator indexCreator = new DynamicIndexCreator(iba);
-        indexCreator.initialize(inputFile, indexCreator.defaultBinSize());
+        final DynamicIndexCreator indexCreator = new DynamicIndexCreator(inputFile, iba);
         return createIndex(inputFile, new FeatureIterator<FEATURE_TYPE, SOURCE_TYPE>(inputFile, codec), indexCreator);
     }
 
+    /**
+     * @param inputFile The file to be indexed.
+     * @param codec Mechanism for reading inputFile.
+     * @param tabixFormat Header fields for TabixIndex to be produced.
+     * @param sequenceDictionary May be null, but if present may reduce memory footprint for index creation.  Features
+     *                           in inputFile must be in the order defined by sequenceDictionary, if it is present.
+     */
+    public static <FEATURE_TYPE extends Feature, SOURCE_TYPE> TabixIndex createTabixIndex(final File inputFile,
+                                                                                     final FeatureCodec<FEATURE_TYPE, SOURCE_TYPE> codec,
+                                                                                     final TabixFormat tabixFormat,
+                                                                                     final SAMSequenceDictionary sequenceDictionary) {
+        final TabixIndexCreator indexCreator = new TabixIndexCreator(sequenceDictionary, tabixFormat);
+        return (TabixIndex)createIndex(inputFile, new FeatureIterator<FEATURE_TYPE, SOURCE_TYPE>(inputFile, codec), indexCreator);
+    }
+
+
 
     private static Index createIndex(final File inputFile, final FeatureIterator iterator, final IndexCreator creator) {
         Feature lastFeature = null;
@@ -330,7 +367,7 @@ public class IndexFactory {
      */
     static class FeatureIterator<FEATURE_TYPE extends Feature, SOURCE> implements CloseableTribbleIterator<Feature> {
         // the stream we use to get features
-        private SOURCE source;
+        private final SOURCE source;
         // the next feature
         private Feature nextFeature;
         // our codec
@@ -363,7 +400,7 @@ public class IndexFactory {
                 final FeatureCodecHeader header = this.codec.readHeader(source);
                 codec.close(source);
                 return header;
-            } catch (IOException e) {
+            } catch (final IOException e) {
                 throw new TribbleException.InvalidHeader("Error reading header " + e.getMessage());
             }
         }
@@ -374,9 +411,9 @@ public class IndexFactory {
                 final PositionalBufferedStream pbs = new PositionalBufferedStream(is);
                 if ( skip > 0 ) pbs.skip(skip);
                 return pbs;
-            } catch (FileNotFoundException e) {
+            } catch (final FileNotFoundException e) {
                 throw new TribbleException.FeatureFileDoesntExist("Unable to open the input file, most likely the file doesn't exist.", inputFile.getAbsolutePath());
-            } catch (IOException e) {
+            } catch (final IOException e) {
                 throw new TribbleException.MalformedFeatureFile("Error initializing stream", inputFile.getAbsolutePath(), e);
             }
         }
@@ -427,7 +464,7 @@ public class IndexFactory {
                 while (nextFeature == null && !codec.isDone(source)) {
                     nextFeature = codec.decodeLoc(source);
                 }
-            } catch (IOException e) {
+            } catch (final IOException e) {
                 throw new TribbleException.MalformedFeatureFile("Unable to read a line from the file", inputFile.getAbsolutePath(), e);
             }
         }
