@@ -40,7 +40,7 @@ import java.util.*;
  *
  * The VariantContext object is a single general class system for representing genetic variation data composed of:
  *
- * * Allele: representing single genetic haplotypes (A, T, ATC, -)
+ * * Allele: representing single genetic haplotypes (A, T, ATC, -) (note that null alleles are used here for illustration; see the Allele class for how to represent indels)
  * * Genotype: an assignment of alleles for each chromosome of a single named sample at a particular locus
  * * VariantContext: an abstract class holding all segregating alleles at a locus as well as genotypes
  *    for multiple individuals containing alleles at that locus
@@ -72,10 +72,10 @@ import java.util.*;
  * A [ref] / T at 10
  * GenomeLoc snpLoc = GenomeLocParser.createGenomeLoc("chr1", 10, 10);
  *
- * - / ATC [ref] from 20-23
+ * A / ATC [ref] from 20-23
  * GenomeLoc delLoc = GenomeLocParser.createGenomeLoc("chr1", 20, 22);
  *
- *  // - [ref] / ATC immediately after 20
+ *  // A [ref] / ATC immediately after 20
  * GenomeLoc insLoc = GenomeLocParser.createGenomeLoc("chr1", 20, 20);
  *
  * === Alleles ===
@@ -86,9 +86,8 @@ import java.util.*;
  *
  * Alleles can be either reference or non-reference
  *
- * Example alleles used here:
+ * Examples of alleles used here:
  *
- *   del = new Allele("-");
  *   A = new Allele("A");
  *   Aref = new Allele("A", true);
  *   T = new Allele("T");
@@ -116,10 +115,8 @@ import java.util.*;
  * VariantContext vc = new VariantContext(name, delLoc, Arrays.asList(ATCref, del));
  * </pre>
  *
- * The only 2 things that distinguishes between a insertion and deletion are the reference allele
- * and the location of the variation.  An insertion has a Null reference allele and at least
- * one non-reference Non-Null allele.  Additionally, the location of the insertion is immediately after
- * a 1-bp GenomeLoc (at say 20).
+ * The only thing that distinguishes between an insertion and deletion is which is the reference allele.
+ * An insertion has a reference allele that is smaller than the non-reference allele, and vice versa for deletions.
  *
  * <pre>
  * VariantContext vc = new VariantContext("name", insLoc, Arrays.asList(delRef, ATC));
@@ -408,8 +405,17 @@ public class VariantContext implements Feature { // to enable tribble integratio
             VariantContextBuilder builder = new VariantContextBuilder(this);
             GenotypesContext newGenotypes = genotypes.subsetToSamples(sampleNames);
 
-            if ( rederiveAllelesFromGenotypes )
-                builder.alleles(allelesOfGenotypes(newGenotypes));
+            if ( rederiveAllelesFromGenotypes ) {
+                Set<Allele> allelesFromGenotypes = allelesOfGenotypes(newGenotypes);
+
+                // ensure original order of genotypes
+                List<Allele> rederivedAlleles = new ArrayList<Allele>(allelesFromGenotypes.size());
+                for (Allele allele : alleles)
+                    if (allelesFromGenotypes.contains(allele))
+                        rederivedAlleles.add(allele);
+
+                builder.alleles(rederivedAlleles);
+            }
             else {
                 builder.alleles(alleles);
             }
@@ -1189,14 +1195,14 @@ public class VariantContext implements Feature { // to enable tribble integratio
             }
 
             if ( getAttribute(VCFConstants.ALLELE_COUNT_KEY) instanceof List ) {
-                Collections.sort(observedACs);
-                List reportedACs = (List)getAttribute(VCFConstants.ALLELE_COUNT_KEY);
-                Collections.sort(reportedACs);
+                final List reportedACs = (List)getAttribute(VCFConstants.ALLELE_COUNT_KEY);
                 if ( observedACs.size() != reportedACs.size() )
                     throw new TribbleException.InternalCodecException(String.format("the Allele Count (AC) tag doesn't have the correct number of values for the record at position %s:%d, %d vs. %d", getChr(), getStart(), reportedACs.size(), observedACs.size()));
                 for (int i = 0; i < observedACs.size(); i++) {
-                    if ( Integer.valueOf(reportedACs.get(i).toString()) != observedACs.get(i) )
-                        throw new TribbleException.InternalCodecException(String.format("the Allele Count (AC) tag is incorrect for the record at position %s:%d, %s vs. %d", getChr(), getStart(), reportedACs.get(i), observedACs.get(i)));
+                    // need to cast to int to make sure we don't have an issue below with object equals (earlier bug) - EB
+                    final int reportedAC = Integer.valueOf(reportedACs.get(i).toString());
+                    if ( reportedAC != observedACs.get(i) )
+                        throw new TribbleException.InternalCodecException(String.format("the Allele Count (AC) tag is incorrect for the record at position %s:%d, %s vs. %d", getChr(), getStart(), reportedAC, observedACs.get(i)));
                 }
             } else {
                 if ( observedACs.size() != 1 )
@@ -1365,6 +1371,13 @@ public class VariantContext implements Feature { // to enable tribble integratio
     }
 
     public String toString() {
+        // Note: passing genotypes to String.format() will implicitly decode the genotypes
+        // This may not be desirable, so don't decode by default
+
+        return genotypes.isLazyWithData() ? toStringUnparsedGenotypes() : toStringDecodeGenotypes();
+    }
+
+    public String toStringDecodeGenotypes() {
         return String.format("[VC %s @ %s Q%s of type=%s alleles=%s attr=%s GT=%s",
                 getSource(), contig + ":" + (start - stop == 0 ? start : start + "-" + stop),
                 hasLog10PError() ? String.format("%.2f", getPhredScaledQual()) : ".",
@@ -1374,6 +1387,16 @@ public class VariantContext implements Feature { // to enable tribble integratio
                 this.getGenotypes());
     }
 
+    private String toStringUnparsedGenotypes() {
+        return String.format("[VC %s @ %s Q%s of type=%s alleles=%s attr=%s GT=%s",
+                getSource(), contig + ":" + (start - stop == 0 ? start : start + "-" + stop),
+                hasLog10PError() ? String.format("%.2f", getPhredScaledQual()) : ".",
+                this.getType(),
+                ParsingUtils.sortList(this.getAlleles()),
+                ParsingUtils.sortedString(this.getAttributes()),
+                ((LazyGenotypesContext)this.genotypes).getUnparsedGenotypeData());
+    }
+
     public String toStringWithoutGenotypes() {
         return String.format("[VC %s @ %s Q%s of type=%s alleles=%s attr=%s",
                 getSource(), contig + ":" + (start - stop == 0 ? start : start + "-" + stop),
