@@ -35,5 +35,6 @@ public enum Options {
     INDEX_ON_THE_FLY,
     DO_NOT_WRITE_GENOTYPES,
     ALLOW_MISSING_FIELDS_IN_HEADER,
-    FORCE_BCF
+    FORCE_BCF,
+    USE_ASYNC_IO            // Turn on or off the use of asynchronous IO for writing output VCF files.
 }
