@@ -34,6 +34,7 @@ import org.broad.tribble.util.ParsingUtils;
 import org.broadinstitute.variant.utils.GeneralUtils;
 import org.broadinstitute.variant.variantcontext.*;
 
+import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
@@ -86,6 +87,12 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
      */
     protected boolean doOnTheFlyModifications = true;
 
+    /**
+     * If non-null, we will replace the sample name read from the VCF header with this sample name. This feature works
+     * only for single-sample VCFs.
+     */
+    protected String remappedSampleName = null;
+
     protected AbstractVCFCodec() {
         super(VariantContext.class);
     }
@@ -165,6 +172,20 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
                 if ( sawFormatTag && sampleNames.size() == 0 )
                     throw new TribbleException.InvalidHeader("The FORMAT field was provided but there is no genotype/sample data");
 
+                // If we're performing sample name remapping and there is exactly one sample specified in the header, replace
+                // it with the remappedSampleName. Throw an error if there are 0 or multiple samples and remapping was requested
+                // for this file.
+                if ( remappedSampleName != null ) {
+                    // We currently only support on-the-fly sample name remapping for single-sample VCFs
+                    if ( sampleNames.isEmpty() || sampleNames.size() > 1 ) {
+                        throw new TribbleException(String.format("Cannot remap sample name to %s because %s samples are specified in the VCF header, and on-the-fly sample name remapping is only supported for single-sample VCFs",
+                                                                 remappedSampleName, sampleNames.isEmpty() ? "no" : "multiple"));
+                    }
+
+                    sampleNames.clear();
+                    sampleNames.add(remappedSampleName);
+                }
+
             } else {
                 if ( str.startsWith(VCFConstants.INFO_HEADER_START) ) {
                     final VCFInfoHeaderLine info = new VCFInfoHeaderLine(str.substring(7), version);
@@ -523,7 +544,7 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
      */
     private static void checkAllele(String allele, boolean isRef, int lineNo) {
         if ( allele == null || allele.length() == 0 )
-            generateException("Empty alleles are not permitted in VCF records", lineNo);
+            generateException(generateExceptionTextForBadAlleleBases(""), lineNo);
 
         if ( GeneralUtils.DEBUG_MODE_ENABLED && MAX_ALLELE_SIZE_BEFORE_WARNING != -1 && allele.length() > MAX_ALLELE_SIZE_BEFORE_WARNING ) {
             System.err.println(String.format("Allele detected with length %d exceeding max size %d at approximately line %d, likely resulting in degraded VCF processing performance", allele.length(), MAX_ALLELE_SIZE_BEFORE_WARNING, lineNo));
@@ -540,13 +561,27 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
                         " convert your file to VCF4 using VCFTools, available at http://vcftools.sourceforge.net/index.html", lineNo);
 
             if (!Allele.acceptableAlleleBases(allele))
-                generateException("Unparsable vcf record with allele " + allele, lineNo);
+                generateException(generateExceptionTextForBadAlleleBases(allele), lineNo);
 
             if ( isRef && allele.equals(VCFConstants.EMPTY_ALLELE) )
                 generateException("The reference allele cannot be missing", lineNo);
         }
     }
 
+    /**
+     * Generates the exception text for the case where the allele string contains unacceptable bases.
+     *
+     * @param allele   non-null allele string
+     * @return non-null exception text string
+     */
+    private static String generateExceptionTextForBadAlleleBases(final String allele) {
+        if ( allele.length() == 0 )
+            return "empty alleles are not permitted in VCF records";
+        if ( allele.contains("[") || allele.contains("]") || allele.contains(":") || allele.contains(".") )
+            return "VCF support for complex rearrangements with breakends has not yet been implemented";
+        return "unparsable vcf record with allele " + allele;
+    }
+
     /**
      * return true if this is a symbolic allele (e.g. <SOMETAG>) or
      * structural variation breakend (with [ or ]), otherwise false
@@ -683,8 +718,8 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
             }
 
             // check to make sure we found a genotype field if our version is less than 4.1 file
-            if ( version != VCFHeaderVersion.VCF4_1 && genotypeAlleleLocation == -1 )
-                generateException("Unable to find the GT field for the record; the GT field is required in VCF4.0");
+            if ( ! version.isAtLeastAsRecentAs(VCFHeaderVersion.VCF4_1) && genotypeAlleleLocation == -1 )
+                generateException("Unable to find the GT field for the record; the GT field is required before VCF4.1");
             if ( genotypeAlleleLocation > 0 )
                 generateException("Saw GT field at position " + genotypeAlleleLocation + ", but it must be at the first position for genotypes when present");
 
@@ -703,13 +738,16 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
         return new LazyGenotypesContext.LazyData(genotypes, header.getSampleNamesInOrder(), header.getSampleNameToOffset());
     }
 
-
-    private final static String[] INT_DECODE_ARRAY = new String[10000];
-    private final static int[] decodeInts(final String string) {
+    private final String[] INT_DECODE_ARRAY = new String[10000];
+    private final int[] decodeInts(final String string) {
         final int nValues = ParsingUtils.split(string, INT_DECODE_ARRAY, ',');
         final int[] values = new int[nValues];
-        for ( int i = 0; i < nValues; i++ )
-            values[i] = Integer.valueOf(INT_DECODE_ARRAY[i]);
+        try {
+            for ( int i = 0; i < nValues; i++ )
+                values[i] = Integer.valueOf(INT_DECODE_ARRAY[i]);
+        } catch (final NumberFormatException e) {
+            return null;
+        }
         return values;
     }
 
@@ -722,6 +760,16 @@ public abstract class AbstractVCFCodec extends AsciiFeatureCodec<VariantContext>
         doOnTheFlyModifications = false;
     }
 
+    /**
+     * Replaces the sample name read from the VCF header with the remappedSampleName. Works
+     * only for single-sample VCFs -- attempting to perform sample name remapping for multi-sample
+     * VCFs will produce an Exception.
+     *
+     * @param remappedSampleName replacement sample name for the sample specified in the VCF header
+     */
+    public void setRemappedSampleName( final String remappedSampleName ) {
+        this.remappedSampleName = remappedSampleName;
+    }
 
     protected void generateException(String message) {
         throw new TribbleException(String.format("The provided VCF file is malformed at approximately line number %d: %s", lineNo, message));
