@@ -24,7 +24,21 @@
 package net.sf.picard.sam;
 
 import net.sf.picard.PicardException;
-import net.sf.samtools.*;
+import net.sf.samtools.BamFileIoUtils;
+import net.sf.samtools.Cigar;
+import net.sf.samtools.CigarElement;
+import net.sf.samtools.CigarOperator;
+import net.sf.samtools.SAMFileHeader;
+import net.sf.samtools.SAMFileReader;
+import net.sf.samtools.SAMFileWriter;
+import net.sf.samtools.SAMFileWriterFactory;
+import net.sf.samtools.SAMProgramRecord;
+import net.sf.samtools.SAMReadGroupRecord;
+import net.sf.samtools.SAMRecord;
+import net.sf.samtools.SAMRecordIterator;
+import net.sf.samtools.SAMSequenceRecord;
+import net.sf.samtools.SAMTag;
+import net.sf.samtools.SamPairUtil;
 import net.sf.samtools.util.CloserUtil;
 import org.testng.Assert;
 import org.testng.annotations.DataProvider;
@@ -32,7 +46,12 @@ import org.testng.annotations.Test;
 
 import java.io.File;
 import java.io.IOException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
 /**
  *  Test for the MergeBamAlignment class
@@ -69,7 +88,8 @@ public class MergeBamAlignmentTest {
     @Test
     public void testMergerWithSupplemental() throws Exception {
         final File outputWithSupplemental = File.createTempFile("mergeWithSupplementalTest", ".sam");
-        outputWithSupplemental.deleteOnExit();
+        System.out.println(outputWithSupplemental.getAbsolutePath());
+        // outputWithSupplemental.deleteOnExit();
         final MergeBamAlignment merger = new MergeBamAlignment();
         merger.UNMAPPED_BAM = unmappedBam;
         merger.ALIGNED_BAM = Arrays.asList(supplementalReadAlignedBam);
@@ -99,15 +119,19 @@ public class MergeBamAlignmentTest {
 
             // This tests that we clip both (a) when the adapter is marked in the unmapped BAM file and
             // (b) when the insert size is less than the read length
-            if (sam.getReadName().equals("both_reads_align_clip_adapter") ||
-                    sam.getReadName().equals("both_reads_align_clip_marked")) {
+            if (sam.getReadName().equals("both_reads_align_clip_marked")) {
                 Assert.assertEquals(sam.getReferenceName(), "chr7");
                 if (sam.getReadNegativeStrandFlag()) {
-                    Assert.assertEquals(sam.getCigarString(), "5S96M", "Incorrect CIGAR string for " +
-                            sam.getReadName());
+                    Assert.assertEquals(sam.getCigarString(), "5S96M", "Incorrect CIGAR string for " + sam.getReadName());
                 } else {
-                    Assert.assertEquals(sam.getCigarString(), "96M5S", "Incorrect CIGAR string for " +
-                            sam.getReadName());
+                    Assert.assertEquals(sam.getCigarString(), "96M5S", "Incorrect CIGAR string for " + sam.getReadName());
+                }
+            }
+            else if (sam.getReadName().equals("both_reads_align_clip_adapter")) {
+                Assert.assertEquals(sam.getReferenceName(), "chr7");
+                if (!sam.getSupplementaryAlignmentFlag()) {
+                    if (sam.getReadNegativeStrandFlag()) Assert.assertEquals(sam.getCigarString(), "5S96M", "Incorrect CIGAR string for " + sam.getReadName());
+                    else Assert.assertEquals(sam.getCigarString(), "96M5S", "Incorrect CIGAR string for " + sam.getReadName());
                 }
             }
             // This tests that we DON'T clip when we run off the end if there are equal to or more than
@@ -133,7 +157,7 @@ public class MergeBamAlignmentTest {
         }
 
         // Make sure that we have the appropriate primary and supplementary reads in the new file
-        Assert.assertEquals(clipAdapterFlags.size(), foundClipAdapterFlags.size());
+        Assert.assertEquals(foundClipAdapterFlags.size(), clipAdapterFlags.size());
         Collections.sort(clipAdapterFlags);
         Collections.sort(foundClipAdapterFlags);
         for (int i = 0; i < clipAdapterFlags.size(); i++) {
@@ -316,9 +340,9 @@ public class MergeBamAlignmentTest {
         final File target = File.createTempFile("target", "bam");
         target.deleteOnExit();
         final SamAlignmentMerger merger = new SamAlignmentMerger(unmapped,  target, fasta, null, true, false,
-                 true, false, Arrays.asList(aligned), 1, null, null, null, null, null,
+                false, Arrays.asList(aligned), 1, null, null, null, null, null, null,
                 Arrays.asList(SamPairUtil.PairOrientation.FR), SAMFileHeader.SortOrder.coordinate,
-                new BestMapqPrimaryAlignmentSelectionStrategy());
+                new BestMapqPrimaryAlignmentSelectionStrategy(), false);
 
         merger.mergeAlignment();
         Assert.assertEquals(sorted, !merger.getForceSort());
@@ -425,7 +449,7 @@ public class MergeBamAlignmentTest {
     @Test(expectedExceptions = {IllegalStateException.class, PicardException.class})
     public void testOldQuerynameSortFails() throws IOException {
 
-        final File merged = File.createTempFile("merged", ".bam");
+        final File merged = File.createTempFile("merged", BamFileIoUtils.BAM_FILE_EXTENSION);
         merged.deleteOnExit();
 
         final MergeBamAlignment merger = new MergeBamAlignment();
